@@ -3,7 +3,9 @@ Definitions of various types of fields. Supports JSON draft4 types.
 """
 import enum
 import re
-from collections import OrderedDict
+from collections import OrderedDict, deque
+from collections.abc import Iterable
+
 from copy import deepcopy
 from functools import reduce
 from decimal import Decimal, InvalidOperation
@@ -21,6 +23,23 @@ from typedpy.structures import (
 )
 
 
+class SerializableField(Field):
+    """
+    An abstract class for a field that has custom serialization or deserialization.
+    can override the method:
+      serialize(self, value),
+      deserialize(self, value)
+
+    These methods are not being used for pickling.
+    """
+
+    def serialize(self, value):
+        return value
+
+    def deserialize(self, value):
+        return value
+
+
 def _map_to_field(item):
     item = item[0] if isinstance(item, (list, tuple)) and len(item) == 1 else item
     if isinstance(item, StructMeta) and not isinstance(item, Field):
@@ -91,6 +110,20 @@ class StructureReference(Field):
 
 
 class ImmutableField(Field):
+    """
+    A mixin that makes a field class immutable.
+    For Example:
+
+     .. code-block:: python
+
+         class MyFieldType(Field): .....
+
+         class MyImmutableFieldType(ImmutableField, MyFieldType): pass
+
+         # that's all you have to do to make MyImmutableFieldType immutable.
+
+    """
+
     _immutable = True
 
 
@@ -187,7 +220,8 @@ class Integer(TypedField, Number):
 
 class DecimalNumber(Number):
     """
-    An extension of :class:`Number` for a Decimal. Accepts anything that can be converted to a Decimal
+    An extension of :class:`Number` for a Decimal. Accepts anything that can be converted to a Decimal.
+    It converts the value to a Decimal.
     """
 
     def __set__(self, instance, value):
@@ -280,6 +314,14 @@ class Function(Field):
         super().__set__(instance, value)
 
 
+class Generator(TypedField):
+    """
+    A Python generator. Not serializable.
+    """
+
+    _ty = type(x for x in [])
+
+
 class Anything(Field):
     """
     A field that can contain anything (similar to "any" in Typescript).
@@ -348,14 +390,7 @@ class PositiveInt(Integer, Positive):
     pass
 
 
-class _ListStruct(list, ImmutableMixin):
-    """
-    This is a useful wrapper for the content of list in an Array field.
-    It ensures that an update of the form:
-     mystruct.my_array[i] = new_val
-    Will not bypass the validation of the Array.
-    """
-
+class _IteratorProxyMixin:
     class ListIteratorProxy:
         def __init__(self, the_list):
             self.the_list = the_list
@@ -367,9 +402,19 @@ class _ListStruct(list, ImmutableMixin):
                 return self.the_list[self.index - 1]
             raise StopIteration
 
-    def __init__(self, array: Field, struct_instance: Structure, mylist):
+
+class _ListStruct(list, ImmutableMixin, _IteratorProxyMixin):
+    """
+    This is a useful wrapper for the content of list in an Array field.
+    It ensures that an update of the form:
+     mystruct.my_array[i] = new_val
+    Will not bypass the validation of the Array.
+    """
+
+    def __init__(self, array: Field, struct_instance: Structure, mylist, name: str):
         self._field_definition = array
         self._instance = struct_instance
+        self._name = name
         super().__init__(self._get_defensive_copy_if_needed(mylist))
 
     def __setitem__(self, key, value):
@@ -384,7 +429,7 @@ class _ListStruct(list, ImmutableMixin):
 
     def __iter__(self):
         if self._is_immutable():
-            return _ListStruct.ListIteratorProxy(self)
+            return _IteratorProxyMixin.ListIteratorProxy(self)
         return super(_ListStruct, self).__iter__()
 
     def append(self, value):
@@ -434,6 +479,7 @@ class _ListStruct(list, ImmutableMixin):
         return {
             "the_instance": self._instance,
             "the_array": self._field_definition,
+            "the_name": self._name,
             "the_values": self[:],
         }
 
@@ -444,14 +490,163 @@ class _ListStruct(list, ImmutableMixin):
             array=deepcopy(self._field_definition),
             struct_instance=memo.get(instance_id, self._instance),
             mylist=vals,
+            name=self._name,
         )
 
     def __setstate__(self, state):
+        self._name = state["the_name"]
         self._field_definition = state["the_array"]
         self._instance = state["the_instance"]
         super().__init__(state["the_values"])
 
 
+class _DequeStruct(deque, ImmutableMixin, _IteratorProxyMixin):
+    """
+    This is a useful wrapper for the content of list in an Deque field.
+    It ensures that an update of the form:
+     mystruct.my_array[i] = new_val
+    Will not bypass the validation of the Array.
+    """
+
+    def __init__(
+        self,
+        deq: Field = None,
+        struct_instance: Structure = None,
+        mydeque=None,
+        name: str = None,
+    ):
+        self._field_definition = deq
+        self._instance = struct_instance
+        self._name = name
+        if mydeque is not None:
+            super().__init__(self._get_defensive_copy_if_needed(mydeque))
+
+    def __setitem__(self, key, value):
+        self._raise_if_immutable()
+        copied = deque(self)
+        copied[key] = value
+        setattr(self._instance, getattr(self._field_definition, "_name", None), copied)
+
+    def __getitem__(self, item):
+        val = super().__getitem__(item)
+        return self._get_defensive_copy_if_needed(val)
+
+    def __iter__(self):
+        if self._is_immutable():
+            return _ListStruct.ListIteratorProxy(self)
+        return super(_DequeStruct, self).__iter__()
+
+    def append(self, value):
+        self._raise_if_immutable()
+        copied = deque(self)
+        copied.append(value)
+        if self._field_definition:  # Python 3.6
+            setattr(
+                self._instance, getattr(self._field_definition, "_name", None), copied
+            )
+        super().append(value)
+
+    def appendleft(self, value):
+        self._raise_if_immutable()
+        copied = deque(self)
+        copied.appendleft(value)
+        if self._field_definition:  # Python 3.6
+            setattr(
+                self._instance, getattr(self._field_definition, "_name", None), copied
+            )
+        super().append(value)
+
+    def extend(self, iterable: Iterable):
+        self._raise_if_immutable()
+        copied = deque(self)
+        copied.extend(iterable)
+        if getattr(self, "_instance", None):
+            setattr(
+                self._instance, getattr(self._field_definition, "_name", None), copied
+            )
+
+    def extendleft(self, iterable: Iterable):
+        self._raise_if_immutable()
+        copied = deque(self)
+        copied.extendleft(iterable)
+        if getattr(self, "_instance", None):
+            setattr(
+                self._instance, getattr(self._field_definition, "_name", None), copied
+            )
+
+    def insert(self, index: int, value):
+        self._raise_if_immutable()
+        copied = deque(self)
+        copied.insert(index, value)
+        setattr(self._instance, getattr(self._field_definition, "_name", None), copied)
+
+    def remove(self, ind):
+        self._raise_if_immutable()
+        copied = deque(self)
+        copied.remove(ind)
+        setattr(self._instance, getattr(self._field_definition, "_name", None), copied)
+
+    def copy(self):
+        copied = deque(self)
+        return deepcopy(copied) if self._is_immutable() else copied
+
+    def clear(self) -> None:
+        self._raise_if_immutable()
+        setattr(self._instance, getattr(self._field_definition, "_name", None), deque())
+
+    def pop(self, *args, **kwargs):
+        self._raise_if_immutable()
+        copied = deque(self)
+        res = copied.pop()
+        setattr(self._instance, getattr(self._field_definition, "_name", None), copied)
+        return res
+
+    def popleft(self):
+        self._raise_if_immutable()
+        copied = deque(self)
+        res = copied.popleft()
+        setattr(self._instance, getattr(self._field_definition, "_name", None), copied)
+        return res
+
+    def rotate(self, n: int) -> None:
+        self._raise_if_immutable()
+        # no need to validate again
+        super(_DequeStruct, self).rotate(n)
+
+    def reverse(self) -> None:
+        self._raise_if_immutable()
+        # no need to validate again
+        super(_DequeStruct, self).reverse()
+
+    def __getstate__(self):
+        return {
+            "the_instance": self._instance,
+            "field_def": self._field_definition,
+            "the_name": self._name,
+            "the_values": deque(self),
+        }
+
+    def __deepcopy__(self, memo={}):
+        vals = [deepcopy(v) for v in self.copy()]
+        instance_id = id(self._instance)
+        return _DequeStruct(
+            deq=deepcopy(self._field_definition),
+            struct_instance=memo.get(instance_id, self._instance),
+            mydeque=vals,
+            name=self._name,
+        )
+
+    def __reduce__(self):
+        res = super().__reduce__()
+        return res[0], res[1], self.__getstate__(), res[3]
+
+    def __setstate__(self, state):
+        self._name = state["the_name"]
+        self._field_definition = state["field_def"]
+        self._instance = state["the_instance"]
+        super().__init__(state["the_values"])
+
+
 class _DictStruct(dict, ImmutableMixin):
     """
     This is a useful wrapper for the content of dict in an Map field.
@@ -462,9 +657,10 @@ class _DictStruct(dict, ImmutableMixin):
     ...will not bypass the validation of the Map.
     """
 
-    def __init__(self, the_map, struct_instance, mydict):
+    def __init__(self, the_map, struct_instance, mydict, name):
         self._field_definition = the_map
         self._instance = struct_instance
+        self._name = name
         super().__init__(mydict)
 
     def __setitem__(self, key, value):
@@ -486,6 +682,16 @@ class _DictStruct(dict, ImmutableMixin):
         copied = super(_DictStruct, self).copy()
         return deepcopy(copied) if self._is_immutable() else copied
 
+    def __deepcopy__(self, memo={}):
+        new_dict = {deepcopy(k): deepcopy(v) for k, v in self.items()}
+        instance_id = id(self._instance)
+        return _DictStruct(
+            the_map=self._field_definition,
+            struct_instance=memo.get(instance_id, self._instance),
+            mydict=new_dict,
+            name=self._name,
+        )
+
     def items(self):
         return (
             (k, self._get_defensive_copy_if_needed(v))
@@ -527,11 +733,13 @@ class _DictStruct(dict, ImmutableMixin):
             "_instance": self._instance,
             "_map": self._field_definition,
             "mydict": self.copy(),
+            "_name": self._name,
         }
 
     def __setstate__(self, state):
         self._field_definition = state["_map"]
         self._instance = state["_instance"]
+        self._name = state["_name"]
         super().__init__(state["mydict"])
 
 
@@ -564,6 +772,18 @@ class _JSONSchemaDraft4ReuseMeta(_FieldMeta):
         return cls([validate_and_get_field(item)])
 
 
+class ContainNestedFieldMixin(Field):
+    def _set_immutable(self, immutable: bool):
+        items = getattr(self, "items", None)
+        super()._set_immutable(immutable)
+        if isinstance(items, Field):
+            items._set_immutable(immutable)
+        elif isinstance(items, (list, tuple)):
+            for item in items:
+                if isinstance(item, Field):
+                    item._set_immutable(immutable)
+
+
 class SizedCollection:
     def __init__(self, *args, minItems=None, maxItems=None, **kwargs):
         self.minItems = minItems
@@ -585,7 +805,9 @@ class SizedCollection:
             )
 
 
-class Set(SizedCollection, TypedField, metaclass=_CollectionMeta):
+class Set(
+    SizedCollection, ContainNestedFieldMixin, TypedField, metaclass=_CollectionMeta
+):
     """
     A set collection. Accepts input of type `set`
 
@@ -595,8 +817,11 @@ class Set(SizedCollection, TypedField, metaclass=_CollectionMeta):
         maxItems(int): optional
             maximal size
         items(:class:`Field` or :class:`Structure`): optional
-            The type of the content, can be a predefined :class:`Structure` or
-            :class:`Field`
+            The type of the content, can be a predefined :class:`Structure`,
+            :class:`Field` or an arbitrary class. In case of an arbitrary
+            class, an implicit Field class will be created for it behind the
+            scenes. Always prefer an Explicit Typedpy  :class:`Structure` or
+            :class:`Field`  if you can.
 
     Examples:
 
@@ -625,6 +850,7 @@ class Set(SizedCollection, TypedField, metaclass=_CollectionMeta):
                 )
             )
         super().__init__(*args, **kwargs)
+        self._set_immutable(getattr(self, "_immutable", False))
 
     def __set__(self, instance, value):
         cls = self.__class__._ty
@@ -634,17 +860,19 @@ class Set(SizedCollection, TypedField, metaclass=_CollectionMeta):
             )
         self.validate_size(value, self._name)
         if self.items is not None:
-            temp_st = Structure()
             setattr(self.items, "_name", self._name)
             res = []
             for val in value:
+                temp_st = Structure()
                 self.items.__set__(temp_st, val)
                 res.append(getattr(temp_st, getattr(self.items, "_name")))
             value = cls(res)
         super().__set__(instance, value)
 
 
-class Map(SizedCollection, TypedField, metaclass=_CollectionMeta):
+class Map(
+    SizedCollection, ContainNestedFieldMixin, TypedField, metaclass=_CollectionMeta
+):
     """
     A map/dictionary collection. Accepts input of type `dict`
 
@@ -692,6 +920,7 @@ class Map(SizedCollection, TypedField, metaclass=_CollectionMeta):
                 )
         self._custom_deep_copy_implementation = True
         super().__init__(*args, **kwargs)
+        self._set_immutable(getattr(self, "_immutable", False))
 
     def __set__(self, instance, value):
         if not isinstance(value, dict):
@@ -699,22 +928,24 @@ class Map(SizedCollection, TypedField, metaclass=_CollectionMeta):
         self.validate_size(value, self._name)
 
         if self.items is not None:
-            temp_st = Structure()
             key_field, value_field = self.items[0], self.items[1]
             setattr(key_field, "_name", self._name + "_key")
             setattr(value_field, "_name", self._name + "_value")
             res = OrderedDict()
 
             for key, val in value.items():
+                temp_st = Structure()
                 key_field.__set__(temp_st, key)
                 value_field.__set__(temp_st, val)
                 res[getattr(temp_st, getattr(key_field, "_name"))] = getattr(
                     temp_st, getattr(value_field, "_name")
                 )
-        super().__set__(instance, _DictStruct(self, instance, value))
+        super().__set__(instance, _DictStruct(self, instance, value, self._name))
 
 
-class Array(SizedCollection, TypedField, metaclass=_CollectionMeta):
+class Array(
+    SizedCollection, ContainNestedFieldMixin, TypedField, metaclass=_CollectionMeta
+):
     """
     An Array field, similar to a list. Supports the properties in JSON schema draft 4.
     Expected input is of type `list`.
@@ -747,6 +978,9 @@ class Array(SizedCollection, TypedField, metaclass=_CollectionMeta):
                 # Let's say we defined a Structure "Person"
                 people = Array[Person]
 
+                # Assume Foo is an arbitrary (non-Typedpy) class
+                foos = Array[Foo]
+
     """
 
     _ty = list
@@ -773,16 +1007,17 @@ class Array(SizedCollection, TypedField, metaclass=_CollectionMeta):
         else:
             self.items = _map_to_field(items)
         super().__init__(*args, **kwargs)
+        self._set_immutable(getattr(self, "_immutable", False))
 
     def __set__(self, instance, value):
         verify_type_and_uniqueness(list, value, self._name, self.uniqueItems)
         self.validate_size(value, self._name)
         if self.items is not None:
             if isinstance(self.items, Field):
-                temp_st = Structure()
                 setattr(self.items, "_name", self._name)
                 res = []
                 for i, val in enumerate(value):
+                    temp_st = Structure()
                     setattr(self.items, "_name", self._name + "_{}".format(str(i)))
                     self.items.__set__(temp_st, val)
                     res.append(getattr(temp_st, getattr(self.items, "_name")))
@@ -811,7 +1046,114 @@ class Array(SizedCollection, TypedField, metaclass=_CollectionMeta):
                 res += value[len(self.items) :]
                 value = res
 
-        super().__set__(instance, _ListStruct(self, instance, value))
+        super().__set__(instance, _ListStruct(self, instance, value, self._name))
+
+
+class Deque(
+    SizedCollection, ContainNestedFieldMixin, TypedField, metaclass=_CollectionMeta
+):
+    """
+    An collections.deque field. Supports the properties in JSON schema draft 4.
+    Expected input is of type `collections.deque`.
+
+    Arguments:
+        minItems(int): optional
+            minimal size
+        maxItems(int): optional
+            maximal size
+        unqieItems(bool): optional
+            are elements required to be unique?
+        additionalItems(bool): optional
+            Relevant in case items parameter is a list of Fields. Is it allowed to have additional
+            elements beyond the ones defined in "items"?
+        items(a :class:`Field` or :class:`Structure`, or a list/tuple of :class:`Field` or :class:`Structure`): optional
+            Describes the fields of the elements.
+            If a items if a :class:`Field`, then it applies to all items.
+            If a items is a list, then every element in the content is expected to be
+            of the corresponding field in items.
+            Examples:
+
+            .. code-block:: python
+
+                names = Deque[String]
+                names = Deque[String(minLengh=3)]
+                names = Deque(minItems=5, items=String)
+                my_record = Deque(items=[String, Integer(minimum=5), String])
+                my_lists = Deque[Array[Integer]]
+                my_structs = Deque[StructureReference(a=Integer, b=Float)]
+                # Let's say we defined a Structure "Person"
+                people = Deque[Person]
+
+                # Assume Foo is an arbitrary (non-Typedpy) class
+                foos = Deque[Foo]
+
+    """
+
+    _ty = deque
+
+    def __init__(
+        self, *args, items=None, uniqueItems=None, additionalItems=None, **kwargs
+    ):
+        """
+        Constructor
+        :param args: pass-through
+        :param items: either a single field, which will be enforced for all elements, or a list
+         of fields which enforce the elements with the correspondent index
+        :param uniqueItems: are elements required to be unique?
+        :param additionalItems: Relevant if "items" is a list. Is it allowed to have additional
+        elements beyond the ones defined in "items"?
+        :param kwargs: pass-through
+        """
+        self.uniqueItems = uniqueItems
+        self.additionalItems = additionalItems
+        if isinstance(items, list):
+            self.items = []
+            for item in items:
+                self.items.append(_map_to_field(item))
+        else:
+            self.items = _map_to_field(items)
+        super().__init__(*args, **kwargs)
+        self._set_immutable(getattr(self, "_immutable", False))
+
+    def __set__(self, instance, value):
+        verify_type_and_uniqueness(deque, value, self._name, self.uniqueItems)
+        self.validate_size(value, self._name)
+        if self.items is not None:
+            if isinstance(self.items, Field):
+                setattr(self.items, "_name", self._name)
+                res = deque()
+                for i, val in enumerate(value):
+                    temp_st = Structure()
+                    setattr(self.items, "_name", self._name + "_{}".format(str(i)))
+                    self.items.__set__(temp_st, val)
+                    res.append(getattr(temp_st, getattr(self.items, "_name")))
+                value = res
+            elif isinstance(self.items, list):
+                additional_properties_forbidden = self.additionalItems is False
+
+                if not getattr(instance, "_skip_validation", False):
+                    if len(self.items) > len(value) or (
+                        additional_properties_forbidden and len(self.items) > len(value)
+                    ):
+                        raise ValueError(
+                            "{}: Got {}; Expected an deque of length {}".format(
+                                self._name, value, len(self.items)
+                            )
+                        )
+                temp_st = Structure()
+                temp_st._skip_validation = getattr(instance, "_skip_validation", False)
+                res = deque()
+                for ind, item in enumerate(self.items):
+                    if ind >= len(value):
+                        continue
+                    setattr(item, "_name", self._name + "_{}".format(str(ind)))
+                    item.__set__(temp_st, value[ind])
+                    res.append(getattr(temp_st, getattr(item, "_name")))
+                for i in range(len(self.items), len(value)):
+                    res.append(value[i])
+                value = res
+
+        super().__set__(instance, _DequeStruct(self, instance, value, self._name))
 
 
 def verify_type_and_uniqueness(the_type, value, name, has_unique_items):
@@ -833,7 +1175,7 @@ def verify_type_and_uniqueness(the_type, value, name, has_unique_items):
             )
 
 
-class Tuple(TypedField, metaclass=_CollectionMeta):
+class Tuple(ContainNestedFieldMixin, TypedField, metaclass=_CollectionMeta):
     """
     A tuple field, supports unique items option.
        Expected input is of type `tuple`.
@@ -853,23 +1195,26 @@ class Tuple(TypedField, metaclass=_CollectionMeta):
 
     .. code-block:: python
 
-        // a is a tuple of exactly 2 strings that are different from each other.
+        # a is a tuple of exactly 2 strings that are different from each other.
         a = Tuple(uniqueItems=True, items = [String, String])
 
-        // b is a tuple of 3: string, string and a number up to 10.
+        # b is a tuple of 3: string, string and a number up to 10.
         b = Tuple(items = [String, String, Number(maximum=10)])
 
-        // c is a tuple of 3: integer, string, float.
+        # c is a tuple of 3: integer, string, float.
         c = Tuple[Integer, String, Float]
 
-        // The following define a tuple of any number of Integers
+        # The following define a tuple of any number of Integers
         d = Tuple[Integer]
 
-        // It can also contain other structures:
-        // Assume we have something like: class Foo(Structure): pass
-        // e is a tuple of any number of Integers or Foo instances
+        # It can also contain other structures:
+        # Assume we have something like: class Foo(Structure): pass
+        # e is a tuple of any number of Integers or Foo instances
         e = Tuple[AnyOf[Integer, Foo]]
 
+        # It can also have arbitrary class
+        class MyCustomClass: pass
+        Tuple[MyCustomClass]
     """
 
     _ty = tuple
@@ -898,6 +1243,7 @@ class Tuple(TypedField, metaclass=_CollectionMeta):
         else:
             raise TypeError("Expected a list/tuple of Fields or a single Field")
         super().__init__(*args, **kwargs)
+        self._set_immutable(getattr(self, "_immutable", False))
 
     def __set__(self, instance, value):
         verify_type_and_uniqueness(tuple, value, self._name, self.uniqueItems)
@@ -1215,6 +1561,11 @@ class NotField(MultiFieldWrapper, Field, metaclass=_JSONSchemaDraft4ReuseMeta):
 
 
 class ImmutableSet(Set, ImmutableField):
+    """
+    An immutable  :class:`Set`. Internally implemented by a Python frozenset, so it does not have
+    any mutation methods. This makes it more developer-friendly.
+    """
+
     _ty = frozenset
 
     def __set__(self, instance, value):
@@ -1228,6 +1579,8 @@ class ImmutableSet(Set, ImmutableField):
             setattr(self.items, "_name", self._name)
             res = set()
             for val in value:
+                if getattr(self, "_immutable", False):
+                    temp_st = Structure()
                 self.items.__set__(temp_st, val)
                 res.add(getattr(temp_st, getattr(self.items, "_name")))
                 value = res
@@ -1236,46 +1589,61 @@ class ImmutableSet(Set, ImmutableField):
 
 
 class ImmutableMap(ImmutableField, Map):
+    """
+    An immutable version of :class:`Map`
+    """
+
     pass
 
 
 class ImmutableArray(ImmutableField, Array):
+    """
+    An immutable version of :class:`Array`
+    """
+
+    pass
+
+
+class ImmutableDeque(ImmutableField, Deque):
+    """
+    An immutable version of :class:`Deque`
+    """
+
     pass
 
 
 class ImmutableString(ImmutableField, String):
+    """
+    An immutable version of :class:`String`
+    """
+
     pass
 
 
 class ImmutableNumber(ImmutableField, Number):
+    """
+    An immutable version of :class:`Number`
+    """
+
     pass
 
 
 class ImmutableInteger(ImmutableField, Integer):
+    """
+    An immutable version of :class:`Integer`
+    """
+
     pass
 
 
 class ImmutableFloat(ImmutableField, Float):
+    """
+    An immutable version of :class:`Float`
+    """
+
     pass
 
 
-class SerializableField(Field):
-    """
-    An abstract class for a field that has custom serialization or deserialization.
-    can override the method:
-      serialize(self, value),
-      deserialize(self, value)
-
-    These methods are not being used for pickling.
-    """
-
-    def serialize(self, value):
-        return value
-
-    def deserialize(self, value):
-        return value
-
-
 class ExceptionField(TypedField, SerializableField):
     """
     As Exception. This is serialized as the string representation of the exception.
