@@ -4,7 +4,7 @@ Structure, Field, StructureReference, ClassReference, TypedField
 """
 import enum
 from copy import deepcopy
-from collections import OrderedDict
+from collections import OrderedDict, deque
 from inspect import Signature, Parameter, signature
 import sys
 import typing
@@ -34,13 +34,15 @@ class ImmutableMixin:
         return deepcopy(value) if self._is_immutable() else value
 
     def _is_immutable(self):
+        if getattr(self._field_definition, "_immutable", False):
+            return True
         instance = getattr(self, "_instance", None)
         return getattr(self._instance, IS_IMMUTABLE, False) if instance else False
 
     def _raise_if_immutable(self):
         if self._is_immutable():
-            name = self._field_definition, "_name", None
-            raise ValueError("{}: Structure is immutable".format(name))
+            name = getattr(self, "_name", None)
+            raise ValueError("{}: Field is immutable".format(name))
 
 
 def make_signature(names, required, additional_properties, bases_params_by_name):
@@ -205,7 +207,14 @@ class Field(metaclass=_FieldMeta):
         if getattr(self, IS_IMMUTABLE, False) and not getattr(
             self, "_custom_deep_copy_implementation", False
         ):
-            instance.__dict__[self._name] = deepcopy(value)
+            try:
+                instance.__dict__[self._name] = deepcopy(value)
+            except TypeError:
+                raise TypeError(
+                    "{} cannot be immutable, as its type does not support pickle.".format(
+                        self._name
+                    )
+                )
         else:
             instance.__dict__[self._name] = value
         if getattr(instance, "_instantiated", False) and not getattr(
@@ -237,6 +246,9 @@ class Field(metaclass=_FieldMeta):
         propst = ". Properties: {}".format(", ".join(props)) if props else ""
         return "<{}{}>".format(name, propst)
 
+    def _set_immutable(self, immutable: bool):
+        self._immutable = immutable
+
 
 # noinspection PyBroadException
 def is_function_returning_field(field_definition_candidate):
@@ -373,9 +385,11 @@ def convert_basic_types(v):
         ImmutableSet,
         AnyOf,
         Anything,
+        Deque,
     )
 
     type_mapping = {
+        deque: Deque,
         int: Integer,
         str: String,
         float: Float,
@@ -407,7 +421,7 @@ def _type_is_generic(v):
         )
         or (
             python_ver_atleast_39
-            and origin in {list, dict, tuple, set, frozenset, typing.Union}
+            and origin in {list, dict, tuple, set, frozenset, deque, typing.Union}
         )
     )
 
